@@ -1,23 +1,33 @@
 import os
+import urllib
 
-from pyramid.response import Response
-from pyramid.response import FileResponse
-from pyramid.view import view_config, view_defaults
 from pyramid.httpexceptions import HTTPFound
-from pyramid.threadlocal import get_current_registry
 from pyramid.i18n import TranslationStringFactory
-from sqlalchemy.orm.exc import NoResultFound
-from skosprovider_sqlalchemy.models import Collection, Concept, LabelType, NoteType
+from pyramid.response import FileResponse
+from pyramid.response import Response
+from pyramid.threadlocal import get_current_registry
+from pyramid.view import view_config
+from pyramid.view import view_defaults
+from skosprovider_sqlalchemy.models import Collection
+from skosprovider_sqlalchemy.models import Concept
+from skosprovider_sqlalchemy.models import LabelType
+from skosprovider_sqlalchemy.models import NoteType
+from sqlalchemy.exc import NoResultFound
 
-from atramhasis.errors import SkosRegistryNotFoundException, ConceptSchemeNotFoundException, ConceptNotFoundException
-from atramhasis.utils import update_last_visited_concepts
-from atramhasis.cache import tree_region, invalidate_scheme_cache, invalidate_cache, list_region
 from atramhasis.audit import audit
+from atramhasis.cache import invalidate_cache
+from atramhasis.cache import invalidate_scheme_cache
+from atramhasis.cache import list_region
+from atramhasis.cache import tree_region
+from atramhasis.errors import ConceptNotFoundException
+from atramhasis.errors import ConceptSchemeNotFoundException
+from atramhasis.errors import SkosRegistryNotFoundException
+from atramhasis.utils import update_last_visited_concepts
 
 
 def labels_to_string(labels, ltype):
     labelstring = ''
-    for label in (l for l in labels if l.labeltype_id == ltype):
+    for label in (label for label in labels if label.labeltype_id == ltype):
         labelstring += label.label + ' (' + label.language_id + '), '
     return labelstring[:-2]
 
@@ -34,8 +44,21 @@ def sort_by_labels(concepts, locale, reverse=False):
                   key=lambda child: child.label(locale).label.lower()
                   ) + [x for x in concepts if not x.label(locale)]
 
+def get_public_conceptschemes(skos_registry):
+    """
+    Get all conceptschemes that are visible through the public UI.
+    """
+    conceptschemes = [
+        {'id': x.get_metadata()['id'],
+         'conceptscheme': x.concept_scheme}
+        for x in skos_registry.get_providers() if not any([not_shown in x.get_metadata()['subject']
+                                                           for not_shown in ['external', 'hidden']])
+        ]
 
-@view_defaults(accept='text/html')
+    return conceptschemes
+
+
+@view_defaults(accept='text/html', request_method='GET')
 class AtramhasisView:
     """
     This object groups HTML views part of the public user interface.
@@ -61,7 +84,7 @@ class AtramhasisView:
     @view_config(name='favicon.ico')
     def favicon_view(self):
         """
-        This view returns the favicon when requested from the web root.
+        Return the favicon when requested from the web root.
         """
         here = os.path.dirname(__file__)
         icon = os.path.join(os.path.dirname(here), 'static', 'img', 'favicon.ico')
@@ -72,49 +95,35 @@ class AtramhasisView:
         )
         return response
 
+
     @view_config(route_name='home', renderer='atramhasis:templates/home.jinja2')
     def home_view(self):
         """
-        This view displays the homepage.
+        Display the homepage.
         """
-        conceptschemes = [
-            {'id': x.get_metadata()['id'],
-             'conceptscheme': x.concept_scheme}
-            for x in self.skos_registry.get_providers() if not any([not_shown in x.get_metadata()['subject']
-                                                                    for not_shown in ['external', 'hidden']])
-            ]
+        return {'conceptschemes': get_public_conceptschemes(self.skos_registry)}
 
-        return {'conceptschemes': conceptschemes}
-
-    @view_config(route_name='conceptschemes', renderer='atramhasis:templates/conceptschemes.jinja2')
+    @view_config(route_name='skosprovider.conceptschemes', renderer='atramhasis:templates/conceptschemes.jinja2')
     def conceptschemes_view(self):
         """
-        This view displays a list of available conceptschemes.
+        Display a list of available conceptschemes.
         """
-        conceptschemes = [
-            {'id': x.get_metadata()['id'],
-             'conceptscheme': x.concept_scheme}
-            for x in self.skos_registry.get_providers() if not any([not_shown in x.get_metadata()['subject']
-                                                                    for not_shown in ['external', 'hidden']])
-            ]
-
-        return {'conceptschemes': conceptschemes}
+        return {'conceptschemes': get_public_conceptschemes(self.skos_registry)}
 
     @audit
-    @view_config(route_name='conceptscheme', renderer='atramhasis:templates/conceptscheme.jinja2')
+    @view_config(route_name='skosprovider.conceptscheme', renderer='atramhasis:templates/conceptscheme.jinja2')
     def conceptscheme_view(self):
         """
-        This view displays conceptscheme details.
+        Display a single conceptscheme.
         """
-        conceptschemes = [
-            {'id': x.get_metadata()['id'],
-             'conceptscheme': x.concept_scheme}
-            for x in self.skos_registry.get_providers() if not any([not_shown in x.get_metadata()['subject']
-                                                                    for not_shown in ['external', 'hidden']])
-            ]
+        conceptschemes = get_public_conceptschemes(self.skos_registry)
 
         scheme_id = self.request.matchdict['scheme_id']
+
         provider = self.request.skos_registry.get_provider(scheme_id)
+        if provider is False:
+            raise ConceptSchemeNotFoundException(scheme_id)
+
         conceptscheme = provider.concept_scheme
         if 'atramhasis.force_display_label_language' in provider.metadata:
             locale = provider.metadata['atramhasis.force_display_label_language']
@@ -137,10 +146,10 @@ class AtramhasisView:
                 'locale': locale}
 
     @audit
-    @view_config(route_name='concept', renderer='atramhasis:templates/concept.jinja2')
+    @view_config(route_name='skosprovider.c', renderer='atramhasis:templates/concept.jinja2')
     def concept_view(self):
         """
-        This view displays the concept details
+        Display all about a single concept or collection.
         """
         conceptschemes = [
             {'id': x.get_metadata()['id'],
@@ -154,7 +163,7 @@ class AtramhasisView:
         provider = self.request.skos_registry.get_provider(scheme_id)
         label = self._read_request_param('label')
         requested_type = self._read_request_param('type')
-        
+
         if not provider:
             raise ConceptSchemeNotFoundException(scheme_id)
         if 'atramhasis.force_display_label_language' in provider.metadata:
@@ -169,7 +178,7 @@ class AtramhasisView:
                 concept_type = "Collection"
             else:
                 return Response('Thing without type: ' + str(c_id), status_int=500)
-            url = self.request.route_url('concept', scheme_id=scheme_id, c_id=c_id)
+            url = self.request.route_url('skosprovider.c', scheme_id=scheme_id, c_id=c_id)
             update_last_visited_concepts(self.request, {'label': c.label(locale).label, 'url': url})
             return {'concept': c, 'conceptType': concept_type, 'scheme_id': scheme_id,
                     'conceptschemes': conceptschemes, 'provider': provider,
@@ -177,10 +186,10 @@ class AtramhasisView:
         except NoResultFound:
             raise ConceptNotFoundException(c_id)
 
-    @view_config(route_name='search_result', renderer='atramhasis:templates/search_result.jinja2')
+    @view_config(route_name='skosprovider.conceptscheme.cs', renderer='atramhasis:templates/search_result.jinja2')
     def search_result(self):
         """
-        This view displays the search results
+        Display search results
         """
         conceptschemes = [
             {'id': x.get_metadata()['id'],
@@ -222,7 +231,7 @@ class AtramhasisView:
     @view_config(route_name='locale')
     def set_locale_cookie(self):
         """
-        This view will set a language cookie
+        Set a language cookie to remember what language a user requested.
         """
         settings = get_current_registry().settings
         default_lang = settings.get('pyramid.default_locale_name')
@@ -244,8 +253,11 @@ class AtramhasisView:
         return response
 
     @audit
-    @view_config(route_name='search_result_export', renderer='csv')
+    @view_config(route_name='skosprovider.conceptscheme.csv', renderer='csv')
     def results_csv(self):
+        """
+        Download search results in CSV format, allowing further processing.
+        """
         header = ['conceptscheme', 'id', 'uri', 'type', 'label', 'prefLabels', 'altLabels', 'definition', 'broader',
                   'narrower', 'related']
         rows = []
@@ -342,14 +354,9 @@ class AtramhasisView:
     @staticmethod
     def create_treeid(parent_tree_id, concept_id):
         if parent_tree_id is None:
-            return str(concept_id)
+            return urllib.parse.quote(str(concept_id), safe="")
         else:
-            return parent_tree_id + "." + str(concept_id)
-
-    @view_config(route_name='scheme_root', renderer='atramhasis:templates/concept.jinja2')
-    def results_tree_html(self):
-        scheme_id = self.request.matchdict['scheme_id']
-        return {'concept': None, 'conceptType': None, 'scheme_id': scheme_id}
+            return parent_tree_id + "|" + urllib.parse.quote(str(concept_id), safe="")
 
 
 @view_defaults(accept='application/json', renderer='json')
